@@ -24,13 +24,11 @@ require_relative 'generic'
 module Async
 	module IO
 		class BasicSocket < Generic
-			wraps ::BasicSocket, :setsockopt, :connect_address, :local_address, :remote_address, :do_not_reverse_lookup, :do_not_reverse_lookup=, :shutdown, :getsockopt, :getsockname, :getpeername, :getpeereid
+			wraps ::BasicSocket, :setsockopt, :connect_address, :close_read, :close_write, :local_address, :remote_address, :do_not_reverse_lookup, :do_not_reverse_lookup=, :shutdown, :getsockopt, :getsockname, :getpeername, :getpeereid
 			
 			wrap_blocking_method :recv, :recv_nonblock
 			wrap_blocking_method :recvmsg, :recvmsg_nonblock
 			
-			wrap_blocking_method :recvfrom, :recvfrom_nonblock
-			
 			wrap_blocking_method :sendmsg, :sendmsg_nonblock
 			wrap_blocking_method :send, :sendmsg_nonblock, invert: false
 			
@@ -39,11 +37,38 @@ module Async
 			end
 		end
 		
-		module ServerSocket
+		module Server
+			def accept_each(task: Task.current)
+				task.annotate "accepting connections #{self.local_address.inspect}"
+				
+				while true
+					self.accept(task: task) do |io, address|
+						yield io, address, task: task
+					end
+				end
+			end
+		end
+		
+		class Socket < BasicSocket
+			wraps ::Socket, :bind, :ipv6only!, :listen
+			
+			wrap_blocking_method :recvfrom, :recvfrom_nonblock
+			
+			include ::Socket::Constants
+			
+			def connect(*args)
+				begin
+					async_send(:connect_nonblock, *args)
+				rescue Errno::EISCONN
+					# We are now connected.
+				end
+			end
+			
+			alias connect_nonblock connect
+			
 			def accept(task: Task.current)
 				peer, address = async_send(:accept_nonblock)
-				
-				wrapper = Socket.new(peer, self.reactor)
+				wrapper = Socket.new(peer, task.reactor)
 				
 				return wrapper, address unless block_given?
 				
@@ -58,30 +83,8 @@ module Async
 				end
 			end
 			
-			def accept_each(task: Task.current)
-				task.annotate "accepting connections #{self.local_address.inspect}"
-				
-				while true
-					self.accept(task: task) do |io, address|
-						yield io, address
-					end
-				end
-			end
-		end
-		
-		class Socket < BasicSocket
-			wraps ::Socket, :bind, :ipv6only!, :listen
-			
-			include ::Socket::Constants
-			include ServerSocket
-			
-			def connect(*args)
-				begin
-					async_send(:connect_nonblock, *args)
-				rescue Errno::EISCONN
-					# We are now connected.
-				end
-			end
+			alias accept_nonblock accept
+			alias sysaccept accept
 			
 			def self.build(*args, task: Task.current)
 				socket = wrapped_klass.new(*args)
@@ -101,11 +104,13 @@ module Async
 			# @param remote_address [Addrinfo] The remote address to connect to.
 			# @param local_address [Addrinfo] The local address to bind to before connecting.
 			# @option protcol [Integer] The socket protocol to use.
-			def self.connect(remote_address, local_address = nil, task: Task.current, **options)
+			def self.connect(remote_address, local_address = nil, reuse_port: false, task: Task.current, **options)
+				# Async.logger.debug(self) {"Connecting to #{remote_address.inspect}"}
+				
 				task.annotate "connecting to #{remote_address.inspect}"
 				
 				wrapper = build(remote_address.afamily, remote_address.socktype, remote_address.protocol, **options) do |socket|
-					socket.setsockopt(::Socket::SOL_SOCKET, ::Socket::SO_REUSEADDR, true)
+					socket.setsockopt(::Socket::SOL_SOCKET, ::Socket::SO_REUSEADDR, reuse_port)
 					
 					if local_address
 						socket.bind(local_address.to_sockaddr)
@@ -138,7 +143,7 @@ module Async
 			# @option protocol [Integer] The socket protocol to use.
 			# @option reuse_port [Boolean] Allow this port to be bound in multiple processes.
 			def self.bind(local_address, protocol: 0, reuse_port: false, task: Task.current, **options, &block)
-				task.annotate "binding to #{local_address.inspect}"
+				Async.logger.debug(self) {"Binding to #{local_address.inspect}"}
 				
 				wrapper = build(local_address.afamily, local_address.socktype, protocol, **options) do |socket|
 					socket.setsockopt(::Socket::SOL_SOCKET, ::Socket::SO_REUSEADDR, true)
@@ -148,10 +153,14 @@ module Async
 				
 				return wrapper unless block_given?
 				
-				begin
-					yield wrapper, task
-				ensure
-					wrapper.close
+				task.async do
+					task.annotate "binding to #{local_address.inspect}"
+					
+					begin
+						yield wrapper, task
+					ensure
+						wrapper.close
+					end
 				end
 			end
 			
@@ -163,10 +172,18 @@ module Async
 					server.accept_each(task: task, &block)
 				end
 			end
+			
+			include Server
+			
+			def self.pair(*args)
+				::Socket.pair(*args).map(&self.method(:new))
+			end
 		end
 		
 		class IPSocket < BasicSocket
 			wraps ::IPSocket, :addr, :peeraddr
+			
+			wrap_blocking_method :recvfrom, :recvfrom_nonblock
 		end
 	end
 end
